use std::path::PathBuf;
// use std::io::BufReader;
use std::thread;
use std::time::Duration;
use std::fs::File;
use std::io::{Cursor, Read};
use std::sync::Arc;
use rodio::Decoder;
use crate::rodio::Source;


// pub fn collect



#[derive(Clone, Default, Debug)]
pub struct Sound {
    pub name: String,
    pub location: PathBuf,
    data: Arc<Vec<u8>>,
    pub volume: f32,
    pub roll: i32,
    pub rate: i32,
    active: bool
}


impl AsRef<[u8]> for Sound {
    fn as_ref(&self) -> &[u8] {
        &self.data
    }
}

impl Sound {

    pub fn new(filename: &str) -> std::io::Result<Sound> {
        let mut buf = Vec::new();
        let mut file = File::open(filename)?;
        file.read_to_end(&mut buf)?;
        Ok(Sound{
            data: Arc::new(buf),
            name: filename.to_string(),
            ..Default::default()
        })
    }
    pub fn cursor(self: &Self) -> Cursor<Sound> {
        Cursor::new(
            Sound {
                data: self.data.clone(),
                ..Default::default()
            })
    }

    pub fn decoder(self: &Self) -> Decoder<Cursor<Sound>> {
        Decoder::new(self.cursor()).unwrap()
    }


    pub fn play(&mut self, device: &rodio::Device) {
        self.active = true;
        rodio::play_raw(&device, self.decoder().convert_samples());
        // self.active = true;

    }
}

#[derive(Clone, Debug)]
pub struct Bar {
    pub repeat: i32,
    pub bpm: i32,
    pub sounds: Vec<Option<Sound>>
}

impl Default for Bar {
    fn default() -> Bar {
        Bar {
            repeat: 0,
            bpm: 120,
            sounds: vec![None; 4]
        }
    }
}


impl Bar {
    pub fn play(&mut self, device: &rodio::Device) {
        let delay = ((60.0 / self.bpm as f32)*1000.0) as u64;
        for _ in 0..self.repeat+1 {
            for sound in &mut self.sounds {
                if let Some(s) = sound {
                    s.play(device);
                }
                thread::sleep(Duration::from_millis(delay));
            }
        }
    }
    pub fn extend(&mut self, slots: usize) {
        self.sounds.resize(self.sounds.len() + slots, None);
    }
    pub fn length(&self) -> usize {
        self.sounds.len()
    }
}

#[derive(Clone, Debug)]
pub struct Track {
    pub bars: Vec<Bar>
}

impl Default for Track {
    fn default() -> Track {
        Track {bars: vec![Bar::default()]}
    }
}

#[derive(Clone, Debug)]
pub struct Timeline {
    pub tracks: Vec<Track>
}

impl Default for Timeline {
    fn default() -> Timeline {
        Timeline {
            tracks: vec![Track::default()]
        }
    }
}